@@ -1,5 +1,5 @@
 import string
-import random
+import secrets
 
 
 class Charset:
@@ -9,6 +9,7 @@ class Charset:
         self.min = min
         self.count = 0
         self._min_ok = False
+        self.timeout = False
         # self._max_ok = False
 
     # @property
@@ -25,16 +26,24 @@ class Password:
         self.length = length
         self.charsets = []
         self.minimum_achieved = False
+        self.password = []
 
     def add_charset(self, charset):
         self.charsets.append(charset)
 
+    def reset_charset(self):
+        for charset in self.charsets:
+            charset.count = 0
+            charset.timeout = False
+
     def generate_password(self):
-        password = []
+        self.password = []
         for _ in range(self.length):
-            password.append(self.__generate_char())
-        random.shuffle(password)
-        return "".join(password)
+            self.password.append(self.__generate_char())
+        # secrets.shuffle(self.password)
+
+        self.reset_charset()
+        return "".join(self.password)
 
     def __generate_char(self):
         if not self.minimum_achieved:
@@ -43,12 +52,23 @@ class Password:
                 self.minimum_achieved = True
                 return self.__generate_char()
 
-            charset = random.choice(pool)
+            charset = secrets.choice(pool)
             charset.count += 1
-        else:
-            charset = random.choice(self.charsets)
 
-        return random.choice(charset.charset)
+        elif len(self.charsets) > 1:
+            # guarantees that will not select the same charset twice in a roll
+            pool = [cs for cs in self.charsets if not cs.timeout]
+            charset = secrets.choice(pool)
+            self.reset_charset()
+            charset.timeout = True
+
+        else:
+            charset = self.charsets[0]
+
+        if (char := secrets.choice(charset.charset)) in self.password:
+            # choice again if char alredy in the password
+            char = secrets.choice(charset.charset)
+        return char
 
 
 def generate(
@@ -59,10 +79,11 @@ def generate(
     letters=True,
     l_upper=True,
     l_lower=True,
-    charset="",
-    **kwargs
+    charsets=[],
+    charset_file="",
+    **kwargs,
 ) -> (str | list):
-    """Generates a random password based on the arguments.
+    """Generates a secrets password based on the arguments.
 
     Args:
         quantity (int, optional): Quantity of passwords to be generate.
@@ -73,7 +94,8 @@ def generate(
         letters (bool, optional): Whether to use letters or not. Defaults to True.
         l_upper (bool, optional): Whether to use uppercase letters or not. Defaults to True.
         l_lower (bool, optional): Whether to use lowercase letters or not. Defaults to True.
-        charset (str, optional): The charset will be used instead of the arguments specification. Defaults to "".
+        charsets (list, optional): The charsets that will be used instead of the arguments specification. Defaults to [].
+        charset_file (str, optional): The charset file will be used instead of the arguments specification. Defaults to "".
 
     Returns:
         str | list: return a list if the quantity is greater than 1
@@ -84,15 +106,30 @@ def generate(
         raise ValueError("quantity must be greater than zero")
     if length < 1:
         raise ValueError("length must be greater than zero")
-    if not punctuation and not digits and not letters:
+    if (
+        not punctuation
+        and not digits
+        and not letters
+        and not charsets
+        and not charset_file
+    ):
         raise ValueError(
-            "at least one of this argument must be True (punctuation, digits, letters)"
+            "at least one of this argument must exists(punctuation, digits, letters, charset, charset_file)"
         )
+    if not isinstance(charsets, list):
+        raise ValueError("charset must be a list")
 
+    encoding = kwargs.get("encoding", "utf-8")
     password_gen = Password(length)
 
-    if charset:
-        password_gen.add_charset(Charset(charset, length))
+    if charsets:
+        for cs in charsets:
+            password_gen.add_charset(Charset(cs, length))
+
+    elif charset_file:
+        with open(charset_file, encoding=encoding) as file:
+            password_gen.add_charset(Charset(file.read(), length))
+
     else:
         if punctuation:
             password_gen.add_charset(Charset(string.punctuation, length))
@@ -112,88 +149,35 @@ def generate(
     return password_gen.generate_password()
 
 
+def test(password: str) -> str:
+    ...
+
+
 def __show_error(msg=""):
     print(f"pwdpy error: {msg}", file=sys.stderr)
 
 
 if __name__ == "__main__":
-    import argparse
     import sys
+    import cli_args
 
-    parser = argparse.ArgumentParser(description="Tools for passwords")
-
-    parser.add_argument(
-        "-l",
-        "--length",
-        help="the length of the password (default: 8)",
-        type=int,
-        default=8,
-    )
-
-    parser.add_argument(
-        "-q",
-        "--quantity",
-        help="quantity of passwords to generate (default: 1)",
-        type=int,
-        default=1,
-    )
-
-    parser.add_argument(
-        "-p",
-        "--punctuation",
-        help="use punctuation characters (default: False)",
-        action="store_true",
-        default=False,
-    )
-
-    parser.add_argument(
-        "-d",
-        "--digits",
-        help="use digits (default: False)",
-        action="store_true",
-        default=False,
-    )
-
-    parser.add_argument(
-        "-le",
-        "--letters",
-        help="use letter (default: False)",
-        action="store_true",
-        default=False,
-    )
-
-    parser.add_argument(
-        "-nu",
-        "--no-upper",
-        help="don't use upper case letters (default: False)",
-        action="store_false",
-        default=True,
-        dest="upper",
-    )
-
-    parser.add_argument(
-        "-nl",
-        "--no-lower",
-        help="don't use lower case letters (default: False)",
-        action="store_false",
-        default=True,
-        dest="lower",
-    )
-
-    args = parser.parse_args()
-
+    args = cli_args.get_args()
     try:
-        print(
-            generate(
-                args.quantity,
-                length=args.length,
-                punctuation=args.punctuation,
-                digits=args.digits,
-                letters=args.letters,
-                l_upper=args.upper,
-                l_lower=args.lower,
-                charset="",
+        if args.command in cli_args.GENERATE:
+            print(
+                generate(
+                    args.quantity,
+                    length=args.length,
+                    punctuation=args.punctuation,
+                    digits=args.digits,
+                    letters=args.letters,
+                    l_upper=args.upper,
+                    l_lower=args.lower,
+                    charset_file=args.charset_file,
+                )
             )
-        )
+        elif args.command in cli_args.TEST:
+            print(test(args.password))
+
     except Exception as error_msg:
-        __show_error(error_msg.args[0])
+        __show_error(error_msg.args[-1])
